import { BaseAI, BaseAIDefaultConfig } from '../base.js';
import { modelInfoAnthropic } from './info.js';
import { AnthropicModel } from './types.js';
/**
 * Anthropic: Default Model options for text generation
 * @export
 */
export const AnthropicDefaultConfig = () => structuredClone({
    model: AnthropicModel.Claude3Haiku,
    ...BaseAIDefaultConfig()
});
export class Anthropic extends BaseAI {
    config;
    constructor({ apiKey, config = AnthropicDefaultConfig(), options }) {
        if (!apiKey || apiKey === '') {
            throw new Error('Anthropic API key not set');
        }
        super({
            name: 'Anthropic',
            apiURL: 'https://api.anthropic.com/v1',
            headers: {
                'anthropic-version': '2023-06-01',
                'anthropic-beta': 'tools-2024-04-04',
                'x-api-key': apiKey
            },
            modelInfo: modelInfoAnthropic,
            models: { model: config.model },
            options,
            supportFor: { functions: true, streaming: true }
        });
        this.config = config;
    }
    getModelConfig() {
        const { config } = this;
        return {
            maxTokens: config.maxTokens,
            temperature: config.temperature,
            topP: config.topP,
            topK: config.topK,
            stream: config.stream
        };
    }
    generateChatReq = (req) => {
        const apiConfig = {
            name: '/messages'
        };
        const messages = req.chatPrompt?.map((msg) => {
            if (msg.role === 'function') {
                return {
                    role: 'user',
                    content: msg.content,
                    tool_use_id: msg.functionId
                };
            }
            return {
                role: msg.role,
                content: msg.content ?? ''
            };
        }) ?? [];
        const tools = req.functions?.map((v) => ({
            name: v.name,
            description: v.description,
            input_schema: v.parameters
        }));
        const reqValue = {
            model: req.modelInfo?.name ?? this.config.model,
            max_tokens: req.modelConfig?.maxTokens ?? this.config.maxTokens,
            stop_sequences: req.modelConfig?.stopSequences ?? this.config.stopSequences,
            temperature: req.modelConfig?.temperature ?? this.config.temperature,
            top_p: req.modelConfig?.topP ?? this.config.topP,
            top_k: req.modelConfig?.topK ?? this.config.topK,
            ...(tools && tools.length > 0 ? { tools } : {}),
            ...(req.identity?.user
                ? {
                    metadata: {
                        user_id: req.identity?.user
                    }
                }
                : {}),
            messages
        };
        return [apiConfig, reqValue];
    };
    generateChatResp = (response) => {
        const err = response;
        if (err.type === 'error') {
            throw new Error(`Anthropic Chat API Error: ${err.error.message}`);
        }
        const resp = response;
        const results = resp.content.map((msg) => {
            let finishReason;
            if (msg.type === 'tool_use') {
                finishReason = 'function_call';
                return {
                    id: msg.id,
                    type: 'function',
                    function: {
                        name: msg.name,
                        arguments: msg.input
                    },
                    content: '',
                    finishReason
                };
            }
            finishReason = mapFinishReason(resp.stop_reason);
            return {
                content: msg.type === 'text' ? msg.text : '',
                role: resp.role,
                id: resp.id,
                finishReason
            };
        });
        const modelUsage = {
            promptTokens: resp.usage.input_tokens,
            completionTokens: resp.usage.output_tokens,
            totalTokens: resp.usage.input_tokens + resp.usage.output_tokens
        };
        return {
            results,
            modelUsage
        };
    };
    generateChatStreamResp = (resp) => {
        let results = [];
        let modelUsage;
        if ('message' in resp) {
            const { message } = resp;
            results = [
                {
                    content: '',
                    id: message.id
                }
            ];
            modelUsage = {
                promptTokens: resp.usage?.input_tokens ?? 0,
                completionTokens: resp.usage?.output_tokens ?? 0,
                totalTokens: (resp.usage?.input_tokens ?? 0) + (resp.usage?.output_tokens ?? 0)
            };
        }
        if ('content_block' in resp) {
            const { content_block: cb } = resp;
            results = [{ content: cb.text }];
        }
        if ('delta' in resp &&
            'text' in resp.delta) {
            const { delta: cb } = resp;
            results = [{ content: cb.text }];
        }
        if ('delta' in resp &&
            'stop_reason' in resp.delta) {
            const { delta } = resp;
            results = [
                { content: '', finishReason: mapFinishReason(delta.stop_reason) }
            ];
            modelUsage = {
                promptTokens: resp.usage?.input_tokens ?? 0,
                completionTokens: resp.usage?.output_tokens ?? 0,
                totalTokens: (resp.usage?.input_tokens ?? 0) + (resp.usage?.output_tokens ?? 0)
            };
        }
        return {
            results,
            modelUsage
        };
    };
}
function mapFinishReason(stopReason) {
    if (!stopReason) {
        return undefined;
    }
    switch (stopReason) {
        case 'stop_sequence':
            return 'stop';
            break;
        case 'max_tokens':
            return 'length';
            break;
        case 'tool_use':
            return 'function_call';
            break;
        case 'end_turn':
            return 'stop';
            break;
        default:
            return 'stop';
    }
}
//# sourceMappingURL=api.js.map