import type { TextModelConfig } from '../types.js';
/**
 * Anthropic: Models for text generation
 * @export
 */
export declare enum AnthropicModel {
    Claude3Opus = "claude-3-opus-20240229",
    Claude3Sonnet = "claude-3-sonnet-20240229",
    Claude3Haiku = "claude-3-haiku-20240307",
    Claude21 = "claude-2.1",
    ClaudeInstant12 = "claude-instant-1.2"
}
/**
 * Anthropic: Model options for text generation
 * @export
 */
export type AnthropicConfig = TextModelConfig & {
    model: AnthropicModel;
};
export type AnthropicChatRequest = {
    model: string;
    messages: {
        role: 'user' | 'assistant' | 'system';
        content: string | {
            type: 'text' | 'image' | 'tool_result';
            text?: string;
            tool_use_id?: string;
            content?: string;
            source?: {
                type: 'base64';
                media_type: string;
                data: string;
            };
        }[];
    }[];
    tools?: {
        name: string;
        description: string;
        input_schema?: object;
    }[];
    max_tokens?: number;
    stop_sequences?: string[];
    stream?: boolean;
    temperature?: number;
    top_p?: number;
    top_k?: number;
    metadata?: {
        user_id: string;
    };
};
export type AnthropicChatResponse = {
    id: string;
    type: 'message';
    role: 'assistant';
    content: {
        id?: string;
        name?: string;
        type: 'text' | 'tool_use';
        text: string;
        input?: string;
    }[];
    model: string;
    stop_reason: 'end_turn' | 'max_tokens' | 'stop_sequence' | 'tool_use';
    stop_sequence?: string;
    usage: {
        input_tokens: number;
        output_tokens: number;
    };
};
export type AnthropicChatError = {
    type: 'error';
    error: {
        type: 'authentication_error';
        message: string;
    };
};
interface AnthropicStreamEvent {
    type: string;
}
export interface MessageStartEvent extends AnthropicStreamEvent {
    message: {
        id: string;
        type: 'message';
        role: 'assistant';
        content: [];
        model: string;
        stop_reason: null | string;
        stop_sequence: null | string;
        usage: {
            input_tokens: number;
            output_tokens: number;
        };
    };
}
export interface ContentBlockStartEvent extends AnthropicStreamEvent {
    index: number;
    content_block: {
        type: 'text';
        text: string;
    };
}
export interface ContentBlockDeltaEvent extends AnthropicStreamEvent {
    index: number;
    delta: {
        type: 'text_delta';
        text: string;
    };
}
interface ContentBlockStopEvent extends AnthropicStreamEvent {
    index: number;
}
export interface MessageDeltaEvent extends AnthropicStreamEvent {
    delta: {
        stop_reason: 'end_turn' | 'max_tokens' | 'stop_sequence' | null;
        stop_sequence: string | null;
        usage: {
            output_tokens: number;
        };
    };
}
type MessageStopEvent = AnthropicStreamEvent;
type PingEvent = AnthropicStreamEvent;
interface ErrorEvent extends AnthropicStreamEvent {
    error: {
        type: 'overloaded_error' | string;
        message: string;
    };
}
type AnthropicStreamEventType = MessageStartEvent | ContentBlockStartEvent | ContentBlockDeltaEvent | ContentBlockStopEvent | MessageDeltaEvent | MessageStopEvent | PingEvent | ErrorEvent;
export interface AnthropicResponseDelta<T> {
    id: string;
    object: 'message';
    model: string;
    events: T[];
    usage?: {
        input_tokens: number;
        output_tokens: number;
    };
}
export type AnthropicChatResponseDelta = AnthropicResponseDelta<AnthropicStreamEventType>;
export {};
