import { BaseAI, BaseAIDefaultConfig, BaseAIDefaultCreativeConfig } from '../base.js';
import { modelInfoHuggingFace } from './info.js';
import { HuggingFaceModel } from './types.js';
/**
 * HuggingFace: Default Model options for text generation
 * @export
 */
export const HuggingFaceDefaultConfig = () => structuredClone({
    model: HuggingFaceModel.MetaLlama270BChatHF,
    ...BaseAIDefaultConfig()
});
/**
 * HuggingFace: Default model options for more creative text generation
 * @export
 */
export const HuggingFaceCreativeConfig = () => structuredClone({
    model: HuggingFaceModel.MetaLlama270BChatHF,
    ...BaseAIDefaultCreativeConfig()
});
/**
 * HuggingFace: AI Service
 * @export
 */
export class HuggingFace extends BaseAI {
    config;
    constructor({ apiKey, config = HuggingFaceDefaultConfig(), options }) {
        if (!apiKey || apiKey === '') {
            throw new Error('HuggingFace API key not set');
        }
        super({
            name: 'HuggingFace',
            apiURL: 'https://api-inference.huggingface.co',
            headers: { Authorization: `Bearer ${apiKey}` },
            modelInfo: modelInfoHuggingFace,
            models: { model: config.model },
            options,
            supportFor: { functions: false, streaming: false }
        });
        this.config = config;
    }
    getModelConfig() {
        const { config } = this;
        return {
            maxTokens: config.maxTokens,
            temperature: config.temperature,
            topP: config.topP,
            topK: config.topK,
            n: config.n,
            presencePenalty: config.presencePenalty
        };
    }
    generateChatReq = (req, 
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    _config) => {
        const model = req.modelInfo?.name ?? this.config.model;
        const functionsList = req.functions
            ? `Functions:\n${JSON.stringify(req.functions, null, 2)}\n`
            : '';
        const prompt = req.chatPrompt
            ?.map((msg) => {
            return `${msg.role}: ${msg.content}`;
        })
            .join('\n');
        const inputs = `${functionsList} ${prompt}`.trim();
        const apiConfig = {
            name: '/models'
        };
        const reqValue = {
            model,
            inputs,
            parameters: {
                max_new_tokens: req.modelConfig?.maxTokens ?? this.config.maxTokens,
                repetition_penalty: req.modelConfig?.presencePenalty ?? this.config.presencePenalty,
                temperature: req.modelConfig?.temperature ?? this.config.temperature,
                top_p: req.modelConfig?.topP ?? this.config.topP,
                top_k: req.modelConfig?.topK ?? this.config.topK,
                return_full_text: this.config.returnFullText,
                num_return_sequences: this.config.n,
                do_sample: this.config.doSample,
                max_time: this.config.maxTime
            },
            options: {
                use_cache: this.config.useCache,
                wait_for_model: this.config.waitForModel
            }
        };
        return [apiConfig, reqValue];
    };
    generateChatResp = (resp) => {
        return {
            results: [
                {
                    content: resp.generated_text
                }
            ]
        };
    };
}
//# sourceMappingURL=api.js.map