import { BaseAI, BaseAIDefaultConfig, BaseAIDefaultCreativeConfig } from '../base.js';
import { modelInfoOpenAI } from './info.js';
import { OpenAIAudioModel, OpenAIEmbedModels, OpenAIModel } from './types.js';
/**
 * OpenAI: Default Model options for text generation
 * @export
 */
export const OpenAIDefaultConfig = () => structuredClone({
    model: OpenAIModel.GPT35Turbo,
    embedModel: OpenAIEmbedModels.TextEmbedding3Small,
    audioModel: OpenAIAudioModel.Whisper1,
    ...BaseAIDefaultConfig()
});
/**
 * OpenAI: Default model options to use the more advanced model
 * @export
 */
export const OpenAIBestConfig = () => structuredClone({
    ...OpenAIDefaultConfig(),
    model: OpenAIModel.GPT4Turbo
});
/**
 * OpenAI: Default model options for more creative text generation
 * @export
 */
export const OpenAICreativeConfig = () => structuredClone({
    model: OpenAIModel.GPT4Turbo,
    embedModel: OpenAIEmbedModels.TextEmbedding3Small,
    audioModel: OpenAIAudioModel.Whisper1,
    ...BaseAIDefaultCreativeConfig()
});
/**
 * OpenAI: Default model options for more fast text generation
 * @export
 */
export const OpenAIFastConfig = () => ({
    ...OpenAIDefaultConfig(),
    model: OpenAIModel.GPT35Turbo
});
/**
 * OpenAI: AI Service
 * @export
 */
export class OpenAI extends BaseAI {
    config;
    streamingUsage;
    constructor({ apiKey, config = OpenAIDefaultConfig(), options, apiURL }) {
        if (!apiKey || apiKey === '') {
            throw new Error('OpenAI API key not set');
        }
        super({
            name: 'OpenAI',
            apiURL: apiURL ? apiURL : 'https://api.openai.com/v1',
            headers: { Authorization: `Bearer ${apiKey}` },
            modelInfo: modelInfoOpenAI,
            models: { model: config.model, embedModel: config.embedModel },
            options,
            supportFor: { functions: true, streaming: true }
        });
        this.config = config;
        this.streamingUsage = options?.streamingUsage ?? true;
    }
    getModelConfig() {
        const { config } = this;
        return {
            maxTokens: config.maxTokens,
            temperature: config.temperature,
            presencePenalty: config.presencePenalty,
            frequencyPenalty: config.frequencyPenalty,
            stopSequences: config.stopSequences,
            topP: config.topP,
            n: config.n,
            stream: config.stream
        };
    }
    generateChatReq = (req, 
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    _config) => {
        const model = req.modelInfo?.name ?? this.config.model;
        if (!req.chatPrompt || req.chatPrompt.length === 0) {
            throw new Error('Chat prompt is empty');
        }
        const apiConfig = {
            name: '/chat/completions',
            headers: {
                ...(req.identity?.organization
                    ? { 'OpenAI-Organization': req.identity?.organization }
                    : null)
            }
        };
        const tools = req.functions?.map((v) => ({
            type: 'function',
            function: {
                name: v.name,
                description: v.description,
                parameters: v.parameters
            }
        }));
        const toolsChoice = !req.functionCall && req.functions && req.functions.length > 0
            ? 'auto'
            : req.functionCall;
        const messages = req.chatPrompt.map((v) => {
            switch (v.role) {
                case 'system':
                    return { role: 'system', content: v.content };
                case 'user':
                    return { role: 'user', content: v.content, name: v.name };
                case 'assistant':
                    return {
                        role: 'assistant',
                        content: v.content,
                        name: v.name,
                        tool_calls: v.functionCalls?.map((v) => ({
                            id: v.id,
                            type: 'function',
                            function: {
                                name: v.function.name,
                                arguments: typeof v.function.arguments === 'object'
                                    ? JSON.stringify(v.function.arguments)
                                    : v.function.arguments
                            }
                        }))
                    };
                case 'function':
                    return {
                        role: 'tool',
                        content: v.content,
                        tool_call_id: v.functionId
                    };
                default:
                    throw new Error('Invalid role');
            }
        });
        const frequencyPenalty = req.modelConfig?.frequencyPenalty ?? this.config.frequencyPenalty;
        const stream = req.modelConfig?.stream ?? this.config.stream;
        const reqValue = {
            model,
            messages,
            response_format: this.config?.responseFormat
                ? { type: this.config?.responseFormat }
                : undefined,
            tools,
            tool_choice: toolsChoice,
            max_tokens: req.modelConfig?.maxTokens ?? this.config.maxTokens ?? 500,
            temperature: req.modelConfig?.temperature ?? this.config.temperature,
            top_p: req.modelConfig?.topP ?? this.config.topP ?? 1,
            n: req.modelConfig?.n ?? this.config.n,
            stop: req.modelConfig?.stopSequences ?? this.config.stop,
            presence_penalty: req.modelConfig?.presencePenalty ?? this.config.presencePenalty,
            logit_bias: this.config.logitBias,
            user: req.identity?.user ?? this.config.user,
            organization: req.identity?.organization,
            ...(frequencyPenalty ? { frequency_penalty: frequencyPenalty } : {}),
            ...(stream && this.streamingUsage
                ? { stream: true, stream_options: { include_usage: true } }
                : {})
        };
        return [apiConfig, reqValue];
    };
    generateEmbedReq = (req) => {
        const model = req.embedModelInfo?.name ?? this.config.embedModel;
        if (!model) {
            throw new Error('Embed model not set');
        }
        if (!req.texts || req.texts.length === 0) {
            throw new Error('Embed texts is empty');
        }
        const apiConfig = {
            name: '/embeddings',
            headers: {
                ...(req.identity?.organization
                    ? { 'OpenAI-Organization': req.identity?.organization }
                    : null)
            }
        };
        const reqValue = {
            model: model,
            input: req.texts
        };
        return [apiConfig, reqValue];
    };
    generateChatResp = (resp) => {
        const { id, usage, choices, error } = resp;
        if (error) {
            throw error;
        }
        const modelUsage = usage
            ? {
                promptTokens: usage.prompt_tokens,
                completionTokens: usage.completion_tokens,
                totalTokens: usage.total_tokens
            }
            : undefined;
        const results = choices.map((choice) => {
            const finishReason = mapFinishReason(choice.finish_reason);
            return {
                id: `${choice.index}`,
                content: choice.message.content,
                functionCalls: choice.message.tool_calls?.map((v) => ({
                    id: v.id,
                    type: 'function',
                    function: {
                        name: v.function.name,
                        arguments: v.function.arguments
                    }
                })),
                finishReason
            };
        });
        return {
            modelUsage,
            results,
            remoteId: id
        };
    };
    generateChatStreamResp = (resp, state) => {
        const { id, usage, choices } = resp;
        const modelUsage = usage
            ? {
                promptTokens: usage.prompt_tokens,
                completionTokens: usage.completion_tokens,
                totalTokens: usage.total_tokens
            }
            : undefined;
        const sstate = state;
        if (!sstate.indexIdMap) {
            sstate.indexIdMap = {};
        }
        const results = choices.map(({ delta: { content, role, tool_calls }, finish_reason }) => {
            const finishReason = mapFinishReason(finish_reason);
            const functionCalls = tool_calls
                ?.map((v) => {
                if (typeof v.id === 'string' &&
                    typeof v.index === 'number' &&
                    !sstate.indexIdMap[v.index]) {
                    sstate.indexIdMap[v.index] = v.id;
                }
                const id = sstate.indexIdMap[v.index];
                if (!id) {
                    return null;
                }
                return {
                    id,
                    type: 'function',
                    function: {
                        name: v.function.name,
                        arguments: v.function.arguments
                    }
                };
            })
                .filter(Boolean);
            return {
                content,
                role: role,
                functionCalls,
                finishReason,
                id
            };
        });
        return {
            results,
            modelUsage
        };
    };
    generateEmbedResp = (resp) => {
        const { data, usage } = resp;
        const modelUsage = usage
            ? {
                promptTokens: usage.prompt_tokens,
                completionTokens: usage.completion_tokens,
                totalTokens: usage.total_tokens
            }
            : undefined;
        return {
            embeddings: data.map((v) => v.embedding),
            modelUsage
        };
    };
}
const mapFinishReason = (finishReason) => {
    switch (finishReason) {
        case 'stop':
            return 'stop';
        case 'length':
            return 'length';
        case 'content_filter':
            return 'error';
        case 'tool_calls':
            return 'function_call';
    }
};
// async _transcribe(
//   file: string,
//   prompt?: string,
//   options?: Readonly<AITranscribeConfig>
// ): Promise<TranscriptResponse> {
//   const res = await this.apiCallWithUpload<
//     OpenAIAudioRequest,
//     OpenAIAudioResponse,
//     OpenAIApiConfig
//   >(
//     this.createAPI(OpenAIApi.Transcribe),
//     generateAudioReq(this.options, prompt, options?.language),
//     file
//   );
//   const { duration, segments } = res;
//   return {
//     duration,
//     segments: segments.map((v) => ({
//       id: v.id,
//       start: v.start,
//       end: v.end,
//       text: v.text,
//     })),
//   };
// }
// export const generateAudioReq = (
//   opt: Readonly<OpenAIConfig>,
//   prompt?: string,
//   language?: string
// ): OpenAIAudioRequest => {
//   if (!opt.audioModel) {
//     throw new Error('OpenAI audio model not set');
//   }
//   return {
//     model: opt.audioModel,
//     prompt,
//     temperature: opt.temperature,
//     language,
//     response_format: 'verbose_json',
//   };
// };
//# sourceMappingURL=api.js.map