import { apiCall } from '../util/apicall.js';
import { BaseDB } from './base.js';
const baseURL = 'https://api.cloudflare.com/client/v4/accounts/';
/**
 * Cloudflare: DB Service
 * @export
 */
export class Cloudflare extends BaseDB {
    apiKey;
    accountId;
    constructor({ apiKey, accountId, fetch, tracer }) {
        if (!apiKey || !accountId) {
            throw new Error('Cloudflare credentials not set');
        }
        super({ name: 'Cloudflare', fetch, tracer });
        this.apiKey = apiKey;
        this.accountId = accountId;
    }
    _upsert = async (req, update, options) => {
        const res = (await apiCall({
            url: new URL(`${this.accountId}/vectorize/indexes/${req.table}/upsert`, baseURL),
            headers: {
                'X-Auth-Key': this.apiKey
            },
            fetch: this.fetch,
            span: options?.span
        }, {
            id: req.id,
            values: req.values,
            namespace: req.namespace,
            metadata: req.metadata
        }));
        if (res.errors) {
            throw new Error(`Cloudflare upsert failed: ${res.errors.map(({ message }) => message).join(', ')}`);
        }
        return {
            ids: res.result.ids
        };
    };
    batchUpsert = async (batchReq, update, options) => {
        if (update) {
            throw new Error('Weaviate does not support batch update');
        }
        if (batchReq.length < 1) {
            throw new Error('Batch request is empty');
        }
        if (!batchReq[0] || !batchReq[0].table) {
            throw new Error('Table name is empty');
        }
        const table = batchReq[0].table;
        const res = (await apiCall({
            url: new URL(`${this.accountId}/vectorize/indexes/${table}/upsert`, baseURL),
            headers: {
                'X-Auth-Key': this.apiKey
            },
            fetch: this.fetch,
            span: options?.span
        }, batchReq.map((req) => ({
            id: req.id,
            values: req.values,
            namespace: req.namespace,
            metadata: req.metadata
        }))));
        if (res.errors) {
            throw new Error(`Cloudflare batch upsert failed: ${res.errors
                .map(({ message }) => message)
                .join(', ')}`);
        }
        return {
            ids: res.result.ids
        };
    };
    query = async (req, options) => {
        const res = (await apiCall({
            url: new URL(`${this.accountId}/vectorize/indexes/${req.table}/query`, baseURL),
            headers: {
                'X-Auth-Key': this.apiKey
            },
            fetch: this.fetch,
            span: options?.span
        }, {
            vector: req.values,
            topK: req.limit || 10,
            returnValues: true
        }));
        if (res.errors) {
            throw new Error(`Cloudflare query failed: ${res.errors.map(({ message }) => message).join(', ')}`);
        }
        const matches = res.result.matches.map(({ id, score, values, metadata }) => ({
            id,
            score,
            values,
            metadata
        }));
        return { matches };
    };
}
//# sourceMappingURL=cloudflare.js.map