import { createHash } from 'crypto';
import { existsSync, mkdirSync, readFileSync, writeFileSync } from 'fs';
import path from 'path';
export class HFDataLoader {
    baseUrl;
    dataFolder;
    constructor() {
        this.baseUrl = 'https://datasets-server.huggingface.co/rows';
        this.dataFolder = path.join(process.cwd(), '.data');
        this.ensureDataFolderExists();
    }
    ensureDataFolderExists() {
        if (!existsSync(this.dataFolder)) {
            mkdirSync(this.dataFolder, { recursive: true });
        }
    }
    async fetchDataFromAPI(url) {
        try {
            const response = await fetch(url);
            if (!response.ok) {
                throw new Error(`Error fetching data: ${response.statusText}`);
            }
            const data = (await response.json());
            if (!data?.rows) {
                throw new Error('Invalid data format');
            }
            return data;
        }
        catch (error) {
            console.error('Error fetching data from API:', error);
            throw error;
        }
    }
    getFilePath(url) {
        // Generate a hash of the URL to use as a filename
        const hash = createHash('md5').update(url).digest('hex');
        return path.join(this.dataFolder, `${hash}.json`);
    }
    // https://datasets-server.huggingface.co/rows?dataset=hotpot_qa&config=distractor&split=train&offset=0&length=100
    async loadData(dataset, split, options) {
        const offset = options?.offset ?? 0;
        const length = options?.length ?? 100;
        const url = `${this.baseUrl}?dataset=${dataset}&config=distractor&split=${split}&offset=${offset}&length=${length}`;
        const filePath = this.getFilePath(url);
        if (existsSync(filePath)) {
            console.log('Loading data from local file.');
            const data = readFileSync(filePath, 'utf8');
            return JSON.parse(data);
        }
        else {
            console.log('Downloading data from API.');
            const data = (await this.fetchDataFromAPI(url));
            writeFileSync(filePath, JSON.stringify(data, null, 2));
            return data;
        }
    }
    async getData({ dataset, split, count, fields, renameMap }) {
        const data = await this.loadData(dataset, split);
        const dataRows = data.rows.slice(0, count);
        return dataRows
            .map((item) => {
            const result = {};
            fields.forEach((field) => {
                const keys = field.split('.');
                // Initial value should match the type of the rows, and be indexable by string
                let value = item.row;
                for (const key of keys) {
                    // Use type assertion to tell TypeScript that value will always be an object that can be indexed with string keys
                    if (Object.prototype.hasOwnProperty.call(value, key)) {
                        value = value[key];
                    }
                }
                if (!value) {
                    return;
                }
                const resultFieldName = renameMap && field in renameMap ? renameMap[field] : field;
                if (!resultFieldName) {
                    throw new Error(`Invalid field name: ${field}`);
                }
                result[resultFieldName] = value;
            });
            return result;
        })
            .filter((v) => Object.keys(v).length !== 0);
    }
}
//# sourceMappingURL=loader.js.map