import { writeFileSync } from 'fs';
import { updateProgressBar } from './util.js';
export class BootstrapFewShot {
    program;
    examples;
    maxRounds;
    maxDemos;
    maxExamples;
    traces = [];
    constructor({ program, examples = [], options }) {
        if (examples.length == 0) {
            throw new Error('No examples found');
        }
        this.maxRounds = options?.maxRounds ?? 3;
        this.maxDemos = options?.maxDemos ?? 4;
        this.maxExamples = options?.maxExamples ?? 16;
        this.program = program;
        this.examples = examples;
    }
    async compileRound(roundIndex, metricFn, options) {
        const st = new Date().getTime();
        const maxDemos = options?.maxDemos ?? this.maxDemos;
        const aiOpt = { modelConfig: { temperature: 0.7 } };
        const examples = randomSample(this.examples, this.maxExamples);
        for (let i = 0; i < examples.length; i++) {
            if (i > 0) {
                aiOpt.modelConfig.temperature = 0.7 + 0.001 * i;
            }
            const ex = examples[i];
            if (!ex) {
                throw new Error('Invalid example');
            }
            const exList = [...examples.slice(0, i), ...examples.slice(i + 1)];
            this.program.setExamples(exList);
            const res = await this.program.forward(ex, aiOpt);
            const success = metricFn({ prediction: res, example: ex });
            if (success) {
                this.traces = [...this.traces, ...this.program.getTraces()];
            }
            const current = i + examples.length * roundIndex;
            const total = examples.length * this.maxRounds;
            const et = new Date().getTime() - st;
            updateProgressBar(current, total, this.traces.length, et, 30, 'Tuning Prompt');
            if (this.traces.length > maxDemos) {
                return;
            }
        }
    }
    async compile(metricFn, options) {
        const maxRounds = options?.maxRounds ?? this.maxRounds;
        this.traces = [];
        for (let i = 0; i < maxRounds; i++) {
            await this.compileRound(i, metricFn, options);
        }
        if (this.traces.length === 0) {
            throw new Error('No demonstrations found. Either provider more examples or improve the existing ones.');
        }
        const demos = groupTracesByKeys(this.traces);
        if (options?.filename) {
            writeFileSync(options.filename, JSON.stringify(demos, null, 2));
        }
        console.log('\n');
        return demos;
    }
}
function groupTracesByKeys(programTraces) {
    const groupedTraces = new Map();
    // Group all traces by their keys
    for (const programTrace of programTraces) {
        if (groupedTraces.has(programTrace.key)) {
            groupedTraces.get(programTrace.key).push(programTrace.trace);
        }
        else {
            groupedTraces.set(programTrace.key, [programTrace.trace]);
        }
    }
    // Convert the Map into an array of ProgramDemos
    const programDemosArray = [];
    groupedTraces.forEach((traces, key) => {
        programDemosArray.push({ traces, key });
    });
    return programDemosArray;
}
const randomSample = (array, n) => {
    // Clone the array to avoid modifying the original array
    const clonedArray = [...array];
    // Shuffle the cloned array
    for (let i = clonedArray.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        const ca_i = clonedArray[i];
        const ca_j = clonedArray[j];
        if (!ca_i || !ca_j) {
            throw new Error('Invalid array elements');
        }
        [clonedArray[i], clonedArray[j]] = [ca_j, ca_i];
    }
    // Return the first `n` items of the shuffled array
    return clonedArray.slice(0, n);
};
//# sourceMappingURL=optimize.js.map