import { createHash } from 'crypto';
import {} from '../text/functions.js';
import { parse } from './parser.js';
export class Signature {
    description;
    inputFields;
    outputFields;
    sigHash;
    sigString;
    constructor(signature) {
        if (typeof signature === 'string') {
            let sig;
            try {
                sig = parse(signature);
            }
            catch (e) {
                throw new Error('invalid signature string: ' + signature);
            }
            this.description = sig.desc;
            this.inputFields = sig.inputs.map((v) => this.parseParsedField(v));
            this.outputFields = sig.outputs.map((v) => this.parseParsedField(v));
            [this.sigHash, this.sigString] = this.updateHash();
        }
        else if (signature instanceof Signature) {
            this.description = signature.getDescription();
            this.inputFields = structuredClone(signature.getInputFields());
            this.outputFields = structuredClone(signature.getOutputFields());
            this.sigHash = signature.hash();
            this.sigString = signature.toString();
        }
        else {
            throw new Error('invalid signature argument: ' + signature);
        }
    }
    parseParsedField = (field) => {
        if (!field.name || field.name.length === 0) {
            throw new Error('Field name is required.');
        }
        const title = this.toTitle(field.name);
        return {
            name: field.name,
            title,
            description: field.desc,
            isOptional: field.isOptional,
            type: field.type ?? { name: 'string', isArray: false }
        };
    };
    parseField = (field) => {
        if (!field.name || field.name.length === 0) {
            throw new Error('Field name is required.');
        }
        const title = !field.title || field.title.length === 0
            ? this.toTitle(field.name)
            : field.title;
        if (field.type && (!field.type.name || field.type.name.length === 0)) {
            throw new Error('Field type name is required: ' + field.name);
        }
        return { ...field, title };
    };
    setDescription = (desc) => {
        this.description = desc;
        this.updateHash();
    };
    addInputField = (field) => {
        this.inputFields.push(this.parseField(field));
        this.updateHash();
    };
    addOutputField = (field) => {
        this.outputFields.push(this.parseField(field));
        this.updateHash();
    };
    setInputFields = (fields) => {
        this.inputFields = fields.map((v) => this.parseField(v));
        this.updateHash();
    };
    setOutputFields = (fields) => {
        this.outputFields = fields.map((v) => this.parseField(v));
        this.updateHash();
    };
    getInputFields = () => this.inputFields;
    getOutputFields = () => this.outputFields;
    getDescription = () => this.description;
    toTitle = (name) => {
        let result = name.replaceAll('_', ' ');
        result = result.replace(/([A-Z]|[0-9]+)/g, ' $1').trim();
        return result.charAt(0).toUpperCase() + result.slice(1);
    };
    toJSONSchema = () => {
        const properties = {};
        const required = [];
        for (const f of this.inputFields) {
            const type = f.type ? f.type.name : 'string';
            if (f.type?.isArray) {
                properties[f.name] = {
                    title: f.title,
                    description: f.description,
                    type: 'array',
                    items: {
                        type: type,
                        description: f.description
                    }
                };
            }
            else {
                properties[f.name] = {
                    description: f.description,
                    type: type
                };
            }
            if (!f.isOptional) {
                required.push(f.name);
            }
        }
        const schema = {
            type: 'object',
            properties: properties,
            required: required
        };
        return schema;
    };
    updateHash = () => {
        this.sigHash = createHash('sha256')
            .update(this.description ?? '')
            .update(JSON.stringify(this.inputFields))
            .update(JSON.stringify(this.outputFields))
            .digest('hex');
        this.sigString = renderSignature(this.description, this.inputFields, this.outputFields);
        return [this.sigHash, this.sigString];
    };
    hash = () => this.sigHash;
    toString = () => this.sigString;
}
function renderField(field) {
    let result = field.name;
    if (field.isOptional) {
        result += '?';
    }
    if (field.type) {
        result += ':' + field.type.name;
        if (field.type.isArray) {
            result += '[]';
        }
    }
    // Check if description exists and append it.
    if (field.description) {
        result += ` "${field.description}"`;
    }
    return result;
}
function renderSignature(description, inputFields, outputFields) {
    // Prepare the description part of the signature.
    const descriptionPart = description ? `"${description}"` : '';
    // Render each input field into a comma-separated list.
    const inputFieldsRendered = inputFields.map(renderField).join(', ');
    // Render each output field into a comma-separated list.
    const outputFieldsRendered = outputFields.map(renderField).join(', ');
    // Combine all parts into the final signature.
    return `${descriptionPart} ${inputFieldsRendered} -> ${outputFieldsRendered}`;
}
//# sourceMappingURL=sig.js.map