export declare const SpanAttributes: {
    LLM_SYSTEM: string;
    LLM_REQUEST_MODEL: string;
    LLM_REQUEST_MAX_TOKENS: string;
    LLM_REQUEST_TEMPERATURE: string;
    LLM_REQUEST_TOP_K: string;
    LLM_REQUEST_FREQUENCY_PENALTY: string;
    LLM_REQUEST_PRESENCE_PENALTY: string;
    LLM_REQUEST_STOP_SEQUENCES: string;
    LLM_REQUEST_LLM_IS_STREAMING: string;
    LLM_REQUEST_TOP_P: string;
    LLM_USAGE_PROMPT_TOKENS: string;
    LLM_USAGE_COMPLETION_TOKENS: string;
    DB_SYSTEM: string;
    DB_TABLE: string;
    DB_NAMESPACE: string;
    DB_ID: string;
    DB_QUERY_TEXT: string;
    DB_VECTOR: string;
    DB_OPERATION_NAME: string;
    DB_VECTOR_QUERY_TOP_K: string;
    DB_QUERY_EMBEDDINGS: string;
    DB_QUERY_RESULT: string;
    DB_QUERY_EMBEDDINGS_VECTOR: string;
    DB_QUERY_RESULT_ID: string;
    DB_QUERY_RESULT_SCORE: string;
    DB_QUERY_RESULT_DISTANCE: string;
    DB_QUERY_RESULT_METADATA: string;
    DB_QUERY_RESULT_VECTOR: string;
    DB_QUERY_RESULT_DOCUMENT: string;
};
export declare const SpanEvents: {
    LLM_PROMPT: string;
};
export declare enum LLMRequestTypeValues {
    COMPLETION = "completion",
    CHAT = "chat",
    RERANK = "rerank",
    UNKNOWN = "unknown"
}
export declare enum SpanKindValues {
    WORKFLOW = "workflow",
    TASK = "task",
    AGENT = "agent",
    TOOL = "tool",
    UNKNOWN = "unknown"
}
export type AttributeValue = string | number | boolean | undefined | null;
export interface Attributes {
    [key: string]: AttributeValue;
}
export interface TimeInput {
    timestamp: number;
}
export interface Exception {
    message: string;
    name: string;
    stack?: string;
}
export interface SpanContext {
    traceId: string;
    spanId: string;
}
export declare enum SpanStatusCode {
    OK = "OK",
    ERROR = "ERROR"
}
export interface SpanStatus {
    code: SpanStatusCode;
    message?: string;
}
export interface SpanStatus {
    code: SpanStatusCode;
    description?: string;
}
export interface Span {
    addEvent(name: string, attributesOrStartTime?: Readonly<Attributes | TimeInput>, startTime?: Readonly<TimeInput>): Span;
    end(endTime?: Readonly<TimeInput>): void;
    isRecording(): boolean;
    recordException(exception: Readonly<Exception>, time?: Readonly<TimeInput>): void;
    setAttribute(key: string, value: AttributeValue): Span;
    setAttributes(attributes: Attributes): Span;
    setStatus(status: Readonly<SpanStatus>): Span;
    spanContext(): SpanContext;
    updateName(name: string): Span;
}
interface Context {
    /**
     * Deletes a value associated with a key from the context.
     * Returns a new context that inherits from the current context but does not contain the value for the specified key.
     * @param key The symbol key for which to clear the value.
     * @returns A new Context instance without the specified key.
     */
    deleteValue(key: symbol): Context;
    /**
     * Retrieves a value from the context using a symbol as the key.
     * @param key The symbol key which identifies a context value.
     * @returns The value associated with the key, if any; otherwise, undefined.
     */
    getValue(key: symbol): unknown;
    /**
     * Sets a value in the context for the specified key.
     * Returns a new context that inherits from the current context with the new key-value pair added.
     * @param key The symbol key for which to set the value.
     * @param value The value to set for the given key.
     * @returns A new Context instance with the updated key-value pair.
     */
    setValue(key: symbol, value: unknown): Context;
}
export declare enum SpanKind {
    INTERNAL = "INTERNAL",
    SERVER = "SERVER",
    CLIENT = "CLIENT",
    PRODUCER = "PRODUCER",
    CONSUMER = "CONSUMER"
}
export interface Link {
    context: SpanContext;
    attributes?: Attributes;
}
export interface SpanOptions {
    /**
     * Optional attributes that can be attached to the Span.
     * @optional
     */
    attributes?: Attributes;
    /**
     * The kind of span, defaults to SpanKind.INTERNAL if not specified.
     * @optional
     */
    kind?: SpanKind;
    /**
     * Links that associate this new Span with other Spans.
     * @optional
     */
    links?: Link[];
    /**
     * Indicates whether the span should be a root span, ignoring any parent span from the context.
     * @optional
     */
    root?: boolean;
    /**
     * A manually specified start time for the span, if required.
     * @optional
     */
    startTime?: TimeInput;
}
export interface Span {
    setStatus(status: Readonly<SpanStatus>): void;
    end(): void;
}
export interface Tracer {
    /**
     * Starts a new Span with the specified options and executes the provided function. The span is automatically closed after the function executes.
     * @param name The name of the span.
     * @param options Span options to apply to the span.
     * @param fn The function to execute within the span's context.
     * @returns The return value of the function.
     */
    startActiveSpan<F extends (span: Span) => unknown>(name: string, options: Readonly<SpanOptions>, fn: F): ReturnType<F>;
    /**
     * Starts a new Span with the specified options and context, then executes the provided function. The span is automatically closed after the function executes.
     * @param name The name of the span.
     * @param options Span options to apply to the span.
     * @param context Context to be used for the span.
     * @param fn The function to execute within the span's context.
     * @returns The return value of the function.
     */
    startActiveSpan<F extends (span: Span) => unknown>(name: string, options: Readonly<SpanOptions>, context: Context, fn: F): ReturnType<F>;
    startSpan(name: string, options?: Readonly<SpanOptions>, context?: Context): Span;
}
export {};
