import { randomUUID } from 'crypto';
export class ModelInfoBuilder {
    info = {};
    setName(name) {
        this.info.name = name;
        return this;
    }
    setCurrency(currency) {
        this.info.currency = currency;
        return this;
    }
    setCharacterIsToken(characterIsToken) {
        this.info.characterIsToken = characterIsToken;
        return this;
    }
    setPromptTokenCostPer1M(promptTokenCostPer1M) {
        this.info.promptTokenCostPer1M = promptTokenCostPer1M;
        return this;
    }
    setCompletionTokenCostPer1M(completionTokenCostPer1M) {
        this.info.completionTokenCostPer1M = completionTokenCostPer1M;
        return this;
    }
    setProvider(provider) {
        this.info.provider = provider;
        return this;
    }
    build() {
        return this.info;
    }
}
export class ModelConfigBuilder {
    config = {};
    setMaxTokens(maxTokens) {
        this.config.maxTokens = maxTokens;
        return this;
    }
    setTemperature(temperature) {
        this.config.temperature = temperature;
        return this;
    }
    setTopP(topP) {
        this.config.topP = topP;
        return this;
    }
    setTopK(topK) {
        this.config.topK = topK;
        return this;
    }
    setN(n) {
        this.config.n = n;
        return this;
    }
    setStream(stream) {
        this.config.stream = stream;
        return this;
    }
    setLogprobs(logprobs) {
        this.config.logprobs = logprobs;
        return this;
    }
    setEcho(echo) {
        this.config.echo = echo;
        return this;
    }
    setPresencePenalty(presencePenalty) {
        this.config.presencePenalty = presencePenalty;
        return this;
    }
    setFrequencyPenalty(frequencyPenalty) {
        this.config.frequencyPenalty = frequencyPenalty;
        return this;
    }
    setBestOf(bestOf) {
        this.config.bestOf = bestOf;
        return this;
    }
    setLogitBias(logitBias) {
        this.config.logitBias = logitBias;
        return this;
    }
    setSuffix(suffix) {
        this.config.suffix = suffix;
        return this;
    }
    build() {
        return this.config;
    }
}
export class TextResponseBuilder {
    response = {};
    setResults(results) {
        if (results) {
            this.response.results = results;
        }
        return this;
    }
    setModelUsage(modelUsage) {
        this.response.modelUsage = modelUsage;
        return this;
    }
    setEmbedModelUsage(embedModelUsage) {
        this.response.embedModelUsage = embedModelUsage;
        return this;
    }
    setRemoteId(remoteId) {
        this.response.remoteId = remoteId;
        return this;
    }
    setModelResponseTime(modelResponseTime) {
        this.response.modelResponseTime = modelResponseTime;
        return this;
    }
    setEmbedModelResponseTime(embedModelResponseTime) {
        this.response.embedModelResponseTime = embedModelResponseTime;
        return this;
    }
    setParsingError(parsingError) {
        this.response.parsingError = parsingError;
        return this;
    }
    setApiError(apiError) {
        this.response.apiError = apiError;
        return this;
    }
    // Include other setter methods for all fields...
    build() {
        return this.response;
    }
}
/*
let response = new TextResponseBuilder()
    .addResults([{text: 'Text', id: '1', finishReason: 'Stop'}])
    .addFunction({name: 'function1', args: '1', result: 'result1'})
    .setModelResponseTime(123)
    .setEmbedModelResponseTime(456)
    .setParsingError({message: 'error', value: 'value'})
    .setApiError({
      message: 'api error',
      status: 400,
      header: {'header1': 'value1'},
      request: {'request1': 'value1'},
      body: {'body1': 'value1'}
    })
    .build();
*/
export class TextRequestBuilder {
    request = {};
    setChatStep(req, modelConfig, modelInfo) {
        const request = { ...req };
        if (!req.modelConfig) {
            request.modelConfig = modelConfig;
        }
        if (!req.modelInfo) {
            request.modelInfo = modelInfo;
        }
        this.request = request;
        return this;
    }
    addChat(chat) {
        const req = this.request;
        if (!req.chatPrompt) {
            req.chatPrompt = [];
        }
        req.chatPrompt.push(chat);
        return this;
    }
    setFunctionCall(functionCall) {
        this.request.functionCall = functionCall;
        return this;
    }
    setEmbedStep(req, modelInfo) {
        const request = { ...req };
        if (!req.embedModelInfo) {
            request.embedModelInfo = modelInfo;
        }
        this.request = request;
        return this;
    }
    setIdentity(identity) {
        this.request.identity = identity;
        return this;
    }
    build() {
        return this.request;
    }
}
export class AITextTraceStepBuilder {
    traceStep = {
        createdAt: new Date().toISOString()
    };
    setTraceId(traceId) {
        this.traceStep.traceId = traceId ?? randomUUID();
        return this;
    }
    setSessionId(sessionId) {
        this.traceStep.sessionId = sessionId;
        return this;
    }
    setRequest(request) {
        this.traceStep.request = request.build();
        return this;
    }
    setResponse(response) {
        this.traceStep.response = response.build();
        return this;
    }
    setModelResponseTime(modelResponseTime) {
        if (!this.traceStep.response) {
            this.traceStep.response = {};
        }
        this.traceStep.response.modelResponseTime = modelResponseTime;
        return this;
    }
    setApiError(apiError) {
        if (!this.traceStep.response) {
            this.traceStep.response = {};
        }
        this.traceStep.response.apiError = apiError;
        return this;
    }
    build() {
        return this.traceStep;
    }
    isStream() {
        return (this.traceStep.request.modelConfig?.stream ?? false);
    }
}
export const sendTrace = async (endpoint, headers, step) => {
    if (!endpoint || endpoint === '') {
        throw new Error('Trace endpoint is required');
    }
    const { traceId, sessionId } = step;
    const apiUrl = new URL(`/api/t/traces`, endpoint);
    const res = await fetch(apiUrl, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
            ...headers
        },
        body: JSON.stringify({ traceId, sessionId, step })
    });
    const respText = await res.text();
    let respVal;
    try {
        const json = JSON.parse(respText);
        respVal = JSON.stringify(json, null, 2);
        if (json.error) {
            throw new Error(`Sending trace to ${apiUrl}:\n${respVal}`);
        }
    }
    catch (e) {
        throw new Error(`Sending trace to ${apiUrl}:\n${respText}`);
    }
    console.log(`Sent trace to ${apiUrl}:\n${respVal}`);
};
export const getMemory = async (endpoint, headers, filter) => {
    const apiUrl = new URL(`/api/t/traces/memory`, endpoint);
    const res = await fetch(apiUrl, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
            ...headers
        },
        body: JSON.stringify({ ...filter, limit: filter.limit ?? 10 })
    });
    const json = (await res.json());
    return json.memory ?? [];
};
//# sourceMappingURL=trace.js.map